(function() {
    // Get application context
    var app = Sammy.apps['#main'];
    var store = app.store;

    /**
     * Services
     *
     */

    // All services status
    app.get('#/services', function (c) {
        c.api('GET', '/services', {}, function(data) {
            var data2 = {
                services: []
            };
            $.each(data, function(k, v) {
                v.name = k;
                // Handlebars want booleans
                v.is_loaded = (v.loaded=='enabled') ? true : false;
                v.is_running = (v.active=='active') ? true : false;
                // Translate status and loaded state
                v.status = y18n.t(v.status);
                v.loaded = y18n.t(v.loaded);
                if (v.active_at == 'unknown')
                {
                    delete v.active_at;
                }
                data2.services.push(v);
            });

            data2.services.sort(function (a, b) {
                if (a.name > b.name) {
                    return 1;
                }
                else if (a.name < b.name) {
                    return -1;
                }
                return 0;
            });

            c.view('service/service_list', data2);
        });
    });

    // Status & actions for a service
    app.get('#/services/:service', function (c) {
        c.api('GET', '/services/'+ c.params['service'], {}, function(data) {
            var data2 = {
                service: data
            };
            data2.service.name = c.params['service'];
            // Handlebars want booleans
            data2.service.is_loaded = (data.loaded=='enabled') ? true : false;
            data2.service.is_running = (data.active=='active') ? true : false;
            // Translate status and loaded state
            data2.service.active = y18n.t(data.active);
            data2.service.loaded = y18n.t(data.loaded);
            if (data.active_at != 'unknown')
            {
                data2.service.active_at = data.active_at;
            }
            else
            {
                data2.service.active_at = 0;
            }
            c.view('service/service_info', data2, function() {

                // Configure behavior for enable/disable and start/stop buttons
                $('button[data-action]').on('click', function() {

                    var service = $(this).data('service');
                    var action = $(this).data('action');

                    c.confirm("Service", y18n.t('confirm_service_' + action, [service]), function(){

                        var method = null,
                            endurl = service;

                        switch (action) {
                            case 'start':
                                method = 'PUT';
                                break;
                            case 'stop':
                                method = 'DELETE';
                                break;
                            case 'enable':
                                method = 'PUT';
                                endurl += '/enable';
                                break;
                            case 'disable':
                                method = 'DELETE';
                                endurl += '/enable';
                                break;
                            default:
                                c.flash('fail', y18n.t('unknown_action', [action]));
                                c.refresh();
                                return;
                        }

                        c.api(method, '/services/'+ endurl, {}, function() { c.refresh(); });
                    });
                });
            });
        });
    });

    // Service log
    app.get('#/services/:service/log', function (c) {
        var params = {
            number: 50
        };
        c.api('GET', '/services/'+ c.params['service'] +'/log', params, function(data) { // ?
            data2 = { 'logs': [], 'name': c.params['service'] };
            $.each(data, function(k, v) {
                data2.logs.push({filename: k, filecontent: v.join('\n')});
            });

            c.view('service/service_log', data2);
        });
    });

})();
